import { neverNull } from "@tutao/tutanota-utils";
import { ProgrammingError } from "./error/ProgrammingError.js";
/**
 * Temporary wrapper to convert from body to details.
 * Should be removed after blob migration is complete.
 */
export class MailWrapper {
    constructor(mail, details) {
        this.mail = mail;
        this.details = details;
    }
    static body(mail, body) {
        return new MailWrapper(mail, { type: "legacy", body });
    }
    static details(mail, details) {
        return new MailWrapper(mail, { type: "details", details });
    }
    isLegacy() {
        return isLegacy(this.details);
    }
    getMail() {
        return this.mail;
    }
    getDetails() {
        if (isLegacy(this.details)) {
            throw new ProgrammingError("can't retrieve details from legacy mail");
        }
        else {
            return this.details.details;
        }
    }
    getBody() {
        if (!isLegacy(this.details)) {
            throw new ProgrammingError("can't retrieve body from new mail");
        }
        return this.details.body;
    }
    getAttachmentIds() {
        return this.mail.attachments;
    }
    getSentDate() {
        if (isLegacy(this.details)) {
            return neverNull(this.mail.sentDate);
        }
        else {
            return this.details.details.sentDate;
        }
    }
    getReplyTos() {
        if (isLegacy(this.details)) {
            return this.mail.replyTos;
        }
        else {
            return this.details.details.replyTos;
        }
    }
    getToRecipients() {
        if (isLegacy(this.details)) {
            return this.mail.toRecipients;
        }
        else {
            return this.details.details.recipients.toRecipients;
        }
    }
    getCcRecipients() {
        if (isLegacy(this.details)) {
            return this.mail.ccRecipients;
        }
        else {
            return this.details.details.recipients.ccRecipients;
        }
    }
    getBccRecipients() {
        if (isLegacy(this.details)) {
            return this.mail.bccRecipients;
        }
        else {
            return this.details.details.recipients.bccRecipients;
        }
    }
    getMailBodyText() {
        if (isLegacy(this.details)) {
            return this.details.body.compressedText ?? this.details.body.text ?? "";
        }
        else {
            return this.details.details.body.compressedText ?? this.details.details.body.text ?? "";
        }
    }
}
function isLegacy(details) {
    return details.type === "legacy";
}
export function isLegacyMail(mail) {
    return mail.mailDetails == null && mail.mailDetailsDraft == null;
}
export function isDetailsDraft(mail) {
    return mail.mailDetailsDraft != null;
}
//# sourceMappingURL=MailWrapper.js.map