//@bundleInto:common-min
import { TutanotaError } from "@tutao/tutanota-error";
export class ConnectionError extends TutanotaError {
    constructor(msg) {
        super("ConnectionError", msg);
    }
}
ConnectionError.CODE = 0;
export class BadRequestError extends TutanotaError {
    constructor(msg) {
        super("BadRequestError", msg);
    }
}
BadRequestError.CODE = 400;
export class NotAuthenticatedError extends TutanotaError {
    constructor(msg) {
        super("NotAuthenticatedError", msg);
    }
}
NotAuthenticatedError.CODE = 401;
export class NotAuthorizedError extends TutanotaError {
    constructor(msg) {
        super("NotAuthorizedError", msg);
    }
}
NotAuthorizedError.CODE = 403;
export class NotFoundError extends TutanotaError {
    constructor(msg) {
        super("NotFoundError", msg);
    }
}
NotFoundError.CODE = 404;
export class MethodNotAllowedError extends TutanotaError {
    constructor(msg) {
        super("MethodNotAllowedError", msg);
    }
}
MethodNotAllowedError.CODE = 405;
export class RequestTimeoutError extends TutanotaError {
    constructor(msg) {
        super("RequestTimeoutError", msg);
    }
}
RequestTimeoutError.CODE = 408;
export class PreconditionFailedError extends TutanotaError {
    constructor(msg, reason) {
        super("PreconditionFailedError", msg);
        this.data = reason;
    }
}
PreconditionFailedError.CODE = 412;
export class LockedError extends TutanotaError {
    constructor(msg) {
        super("LockedError", msg);
    }
}
LockedError.CODE = 423;
export class TooManyRequestsError extends TutanotaError {
    constructor(msg) {
        super("TooManyRequestsError", msg);
    }
}
TooManyRequestsError.CODE = 429;
export class SessionExpiredError extends TutanotaError {
    constructor(msg) {
        super("SessionExpiredError", msg);
    }
}
SessionExpiredError.CODE = 440;
export class AccessDeactivatedError extends TutanotaError {
    constructor(msg) {
        super("AccessDeactivatedError", msg);
    }
}
AccessDeactivatedError.CODE = 470;
/** External users only, related to password changes. */
export class AccessExpiredError extends TutanotaError {
    constructor(msg) {
        super("AccessExpiredError", msg);
    }
}
AccessExpiredError.CODE = 471;
export class AccessBlockedError extends TutanotaError {
    constructor(msg) {
        super("AccessBlockedError", msg);
    }
}
AccessBlockedError.CODE = 472;
export class InvalidDataError extends TutanotaError {
    constructor(msg) {
        super("InvalidDataError", msg);
    }
}
InvalidDataError.CODE = 473;
export class InvalidSoftwareVersionError extends TutanotaError {
    constructor(msg) {
        super("InvalidSoftwareVersionError", msg);
    }
}
InvalidSoftwareVersionError.CODE = 474;
export class LimitReachedError extends TutanotaError {
    constructor(msg) {
        super("LimitReachedError", msg);
    }
}
LimitReachedError.CODE = 475;
export class InternalServerError extends TutanotaError {
    constructor(msg) {
        super("InternalServerError", msg);
    }
}
InternalServerError.CODE = 500;
export class BadGatewayError extends TutanotaError {
    constructor(msg) {
        super("BadGatewayError", msg);
    }
}
BadGatewayError.CODE = 502;
export class ServiceUnavailableError extends TutanotaError {
    constructor(msg) {
        super("ServiceUnavailableError", msg);
    }
}
ServiceUnavailableError.CODE = 503;
export class InsufficientStorageError extends TutanotaError {
    constructor(msg) {
        super("InsufficientStorageError", msg);
    }
}
InsufficientStorageError.CODE = 507;
export class ResourceError extends TutanotaError {
    constructor(msg) {
        super("ResourceError", msg);
    }
}
export class PayloadTooLargeError extends TutanotaError {
    constructor(msg) {
        super("PayloadTooLargeError", msg);
    }
}
PayloadTooLargeError.CODE = 413;
/**
 * Attention: When adding an Error also add it in WorkerProtocol.ErrorNameToType.
 */
export function handleRestError(errorCode, path, errorId, precondition) {
    let message = `${errorCode}: ${errorId ? errorId + " " : ""}${precondition ? precondition + " " : ""}${path}`;
    switch (errorCode) {
        case ConnectionError.CODE:
            return new ConnectionError(message);
        case BadRequestError.CODE:
            return new BadRequestError(message);
        case NotAuthenticatedError.CODE:
            return new NotAuthenticatedError(message);
        case NotAuthorizedError.CODE:
            return new NotAuthorizedError(message);
        case NotFoundError.CODE:
            return new NotFoundError(message);
        case MethodNotAllowedError.CODE:
            return new MethodNotAllowedError(message);
        case PreconditionFailedError.CODE:
            return new PreconditionFailedError(message, precondition ?? null);
        case LockedError.CODE:
            return new LockedError(message);
        case TooManyRequestsError.CODE:
            return new TooManyRequestsError(message);
        case SessionExpiredError.CODE:
            return new SessionExpiredError(message);
        case AccessDeactivatedError.CODE:
            return new AccessDeactivatedError(message);
        case AccessExpiredError.CODE:
            return new AccessExpiredError(message);
        case AccessBlockedError.CODE:
            return new AccessBlockedError(message);
        case InvalidDataError.CODE:
            return new InvalidDataError(message);
        case InvalidSoftwareVersionError.CODE:
            return new InvalidSoftwareVersionError(message);
        case LimitReachedError.CODE:
            return new LimitReachedError(message);
        case InternalServerError.CODE:
            return new InternalServerError(message);
        case BadGatewayError.CODE:
            return new BadGatewayError(message);
        case ServiceUnavailableError.CODE:
            return new ServiceUnavailableError(message);
        case InsufficientStorageError.CODE:
            return new InsufficientStorageError(message);
        case PayloadTooLargeError.CODE:
            return new PayloadTooLargeError(message);
        case RequestTimeoutError.CODE:
            return new RequestTimeoutError(message);
        default:
            return new ResourceError(message);
    }
}
//# sourceMappingURL=RestError.js.map