/** transport impl for communication between the main thread and another browsing context opened in an iframe or another tab */
export class CrossOriginTransport {
    /**
     * @param targetWindow the window object to post / receive messages to, obtained from iframe.contentWindow or window.opener
     * @param targetOrigin the exact origin of targetWindow
     * */
    constructor(targetWindow, targetOrigin) {
        this.targetWindow = targetWindow;
        this.targetOrigin = targetOrigin;
        this.handleMessage = (event) => {
            if (event.source !== this.targetWindow) {
                console.log("got message not from expected thing?");
                return;
            }
            if (event.origin != this.targetOrigin) {
                console.log("got message not from expected origin?");
                return;
            }
            this.handler?.(event.data);
        };
    }
    postMessage(message) {
        return this.targetWindow.postMessage(message, this.targetOrigin);
    }
    setMessageHandler(handler) {
        this.handler = handler;
        window.addEventListener("message", this.handleMessage);
    }
    dispose() {
        window.removeEventListener("message", this.handleMessage);
    }
}
//# sourceMappingURL=CrossOriginTransport.js.map