import { defer } from "@tutao/tutanota-utils";
/** Listener for the login events from the worker side. */
export class PageContextLoginListener {
    constructor(secondFactorHandler) {
        this.secondFactorHandler = secondFactorHandler;
        this.loginPromise = defer();
        this.fullLoginFailed = false;
    }
    /** e.g. after temp logout */
    reset() {
        this.loginPromise = defer();
        this.fullLoginFailed = false;
    }
    waitForFullLogin() {
        return this.loginPromise.promise;
    }
    /**
     * Full login reached: any network requests can be made
     */
    async onFullLoginSuccess() {
        this.fullLoginFailed = false;
        this.loginPromise.resolve();
    }
    /**
     * call when the login fails for invalid session or other reasons
     */
    async onLoginFailure(reason) {
        this.fullLoginFailed = true;
        if (reason === 0 /* LoginFailReason.SessionExpired */) {
            const { reloginForExpiredSession } = await import("../../misc/ErrorHandlerImpl.js");
            await reloginForExpiredSession();
        }
    }
    /**
     * call when retrying full login
     */
    onRetryLogin() {
        this.fullLoginFailed = false;
    }
    /**
     * Shows a dialog with possibility to use second factor and with a message that the login can be approved from another client.
     */
    onSecondFactorChallenge(sessionId, challenges, mailAddress) {
        return this.secondFactorHandler.showSecondFactorAuthenticationDialog(sessionId, challenges, mailAddress);
    }
    /**
     * true if the last full login attempt failed
     * may revert to false when retrying.
     */
    getFullLoginFailed() {
        return this.fullLoginFailed;
    }
}
//# sourceMappingURL=PageContextLoginListener.js.map