import { ARGON2ID_ITERATIONS, ARGON2ID_KEY_LENGTH, ARGON2ID_MEMORY_IN_KiB, ARGON2ID_PARALLELISM, generateKeyFromPassphraseArgon2id, uint8ArrayToBitArray, } from "@tutao/tutanota-crypto";
import { LazyLoaded, stringToUtf8Uint8Array } from "@tutao/tutanota-utils";
import { assertWorkerOrNode } from "../../common/Env.js";
assertWorkerOrNode();
/**
 * WebAssembly implementation of Argon2id
 */
export class WASMArgon2idFacade {
    constructor() {
        // loads argon2 WASM
        this.argon2 = new LazyLoaded(async () => {
            const wasm = fetch("wasm/argon2.wasm");
            if (WebAssembly.instantiateStreaming) {
                return (await WebAssembly.instantiateStreaming(wasm)).instance.exports;
            }
            else {
                // Fallback if the client does not support instantiateStreaming
                const buffer = await (await wasm).arrayBuffer();
                return (await WebAssembly.instantiate(buffer)).instance.exports;
            }
        });
    }
    async generateKeyFromPassphrase(passphrase, salt) {
        return generateKeyFromPassphraseArgon2id(await this.argon2.getAsync(), passphrase, salt);
    }
}
/**
 * Native implementation of Argon2id
 */
export class NativeArgon2idFacade {
    constructor(nativeCryptoFacade) {
        this.nativeCryptoFacade = nativeCryptoFacade;
    }
    async generateKeyFromPassphrase(passphrase, salt) {
        const hash = await this.nativeCryptoFacade.argon2idHashRaw(stringToUtf8Uint8Array(passphrase), salt, ARGON2ID_ITERATIONS, ARGON2ID_MEMORY_IN_KiB, ARGON2ID_PARALLELISM, ARGON2ID_KEY_LENGTH);
        return uint8ArrayToBitArray(hash);
    }
}
//# sourceMappingURL=Argon2idFacade.js.map