import { parseCalendarEvents, parseICalendar } from "./CalendarParser.js";
import { utf8Uint8ArrayToString } from "@tutao/tutanota-utils";
import { getTimeZone } from "../date/CalendarUtils.js";
import { ParserError } from "../../misc/parsing/ParserCombinator.js";
/** given an ical datafile, get the parsed calendar events with their alarms as well as the ical method */
export function parseCalendarFile(file) {
    try {
        const stringData = utf8Uint8ArrayToString(file.data);
        return parseCalendarStringData(stringData, getTimeZone());
    }
    catch (e) {
        if (e instanceof ParserError) {
            throw new ParserError(e.message, file.name);
        }
        else {
            throw e;
        }
    }
}
/** importer internals exported for testing */
export function parseCalendarStringData(value, zone) {
    const tree = parseICalendar(value);
    return parseCalendarEvents(tree, zone);
}
//# sourceMappingURL=CalendarImporter.js.map