import m from "mithril";
import { IconButton } from "../../gui/base/IconButton.js";
import { BaseMobileHeader } from "../../gui/BaseMobileHeader.js";
import { OfflineIndicator } from "../../gui/base/OfflineIndicator.js";
import { ProgressBar } from "../../gui/base/ProgressBar.js";
import { CalendarViewType, getIconForViewType } from "../gui/CalendarGuiUtils.js";
import { MobileHeaderMenuButton, MobileHeaderTitle } from "../../gui/MobileHeader.js";
import { attachDropdown } from "../../gui/base/Dropdown.js";
import { styles } from "../../gui/styles.js";
import { theme } from "../../gui/theme.js";
import { TodayIconButton } from "./TodayIconButton.js";
import { ExpanderButton } from "../../gui/base/Expander.js";
/**
 * A special header that is used instead of {@link MobileHeader} but just for calendar.
 */
export class CalendarMobileHeader {
    view({ attrs }) {
        return m(BaseMobileHeader, {
            left: m(MobileHeaderMenuButton, { newsModel: attrs.newsModel, backAction: () => attrs.viewSlider.focusPreviousColumn() }),
            center: m(MobileHeaderTitle, {
                title: attrs.showExpandIcon
                    ? m(ExpanderButton, {
                        label: () => attrs.navConfiguration.title,
                        isUnformattedLabel: true,
                        style: {
                            "padding-top": "inherit",
                            height: "inherit",
                            "min-height": "inherit",
                            "text-decoration": "none",
                        },
                        expanded: attrs.isDaySelectorExpanded,
                        color: theme.content_fg,
                        isBig: true,
                        isPropagatingEvents: true,
                        onExpandedChange: () => { },
                    })
                    : attrs.navConfiguration.title,
                bottom: m(OfflineIndicator, attrs.offlineIndicatorModel.getCurrentAttrs()),
                onTap: attrs.onTap,
            }),
            right: [
                ...(styles.isDesktopLayout() || styles.isTwoColumnLayout() ? [attrs.navConfiguration.back, attrs.navConfiguration.forward] : []),
                m(TodayIconButton, {
                    click: attrs.onToday,
                }),
                this.renderViewSelector(attrs),
                m(IconButton, {
                    icon: "Add" /* Icons.Add */,
                    title: "newEvent_action",
                    click: attrs.onCreateEvent,
                }),
            ],
            injections: m(ProgressBar, { progress: attrs.offlineIndicatorModel.getProgress() }),
        });
    }
    renderViewSelector(attrs) {
        return m(IconButton, attachDropdown({
            mainButtonAttrs: {
                icon: getIconForViewType(attrs.viewType),
                title: "view_label",
            },
            childAttrs: () => {
                const calendarViewValues = [
                    {
                        name: "agenda_label",
                        value: CalendarViewType.AGENDA,
                    },
                    {
                        name: "day_label",
                        value: CalendarViewType.DAY,
                    },
                    {
                        name: "week_label",
                        value: CalendarViewType.WEEK,
                    },
                    {
                        name: "month_label",
                        value: CalendarViewType.MONTH,
                    },
                ];
                return calendarViewValues.map(({ name, value }) => ({
                    label: name,
                    selected: value === attrs.viewType,
                    icon: getIconForViewType(value),
                    click: () => attrs.onViewTypeSelected(value),
                }));
            },
        }));
    }
}
//# sourceMappingURL=CalendarMobileHeader.js.map