import m from "mithril";
import { size } from "../../gui/size";
import { ListColumnWrapper } from "../../gui/ListColumnWrapper";
import { assertMainOrNode } from "../../api/common/Env";
import { List } from "../../gui/base/List.js";
import { ContactRow } from "./ContactRow.js";
import ColumnEmptyMessageBox from "../../gui/base/ColumnEmptyMessageBox.js";
import { theme } from "../../gui/theme.js";
import { styles } from "../../gui/styles.js";
assertMainOrNode();
export class ContactListView {
    constructor() {
        this.contactViewModel = null;
        this.renderConfig = {
            itemHeight: size.list_row_height,
            multiselectionAllowed: 1 /* MultiselectMode.Enabled */,
            swipe: null,
            createElement: (dom) => {
                return new KindaContactRow(dom, (item) => this.contactViewModel?.listModel.onSingleExclusiveSelection(item));
            },
        };
    }
    view({ attrs: { contactViewModel, onSingleSelection } }) {
        this.contactViewModel = contactViewModel;
        return m(ListColumnWrapper, {
            headerContent: null,
        }, contactViewModel.listModel.isEmptyAndDone()
            ? m(ColumnEmptyMessageBox, {
                color: theme.list_message_bg,
                message: "noContacts_msg",
                icon: "Contacts" /* BootIcons.Contacts */,
            })
            : m(List, {
                renderConfig: this.renderConfig,
                state: contactViewModel.listState(),
                // should not be called anyway
                onLoadMore: () => { },
                onRetryLoading: () => {
                    contactViewModel.listModel.retryLoading();
                },
                onSingleSelection: (item) => {
                    contactViewModel.listModel.onSingleSelection(item);
                    onSingleSelection();
                },
                onSingleTogglingMultiselection: (item) => {
                    contactViewModel.listModel.onSingleInclusiveSelection(item, styles.isSingleColumnLayout());
                },
                onRangeSelectionTowards: (item) => {
                    contactViewModel.listModel.selectRangeTowards(item);
                },
                onStopLoading() {
                    contactViewModel.listModel.stopLoading();
                },
            }));
    }
}
export class KindaContactRow {
    constructor(dom, onToggleSelection) {
        this.entity = null;
        this.cr = new ContactRow(onToggleSelection);
        this.domElement = dom;
        m.render(dom, this.cr.render());
    }
    update(item, selected, multiselect) {
        this.entity = item;
        this.cr.update(item, selected, multiselect);
    }
    render() {
        return this.cr.render();
    }
}
//# sourceMappingURL=ContactListView.js.map