import { lang } from "../../misc/LanguageViewModel";
export class DesktopIntegratorDarwin {
    constructor(electron) {
        this._electron = electron;
    }
    isAutoLaunchEnabled() {
        return Promise.resolve(this._electron.app.getLoginItemSettings().openAtLogin);
    }
    enableAutoLaunch() {
        return this.isAutoLaunchEnabled().then((enabled) => {
            if (!enabled)
                this._electron.app.setLoginItemSettings({
                    openAtLogin: true,
                });
        });
    }
    disableAutoLaunch() {
        return this.isAutoLaunchEnabled().then((enabled) => {
            if (enabled)
                this._electron.app.setLoginItemSettings({
                    openAtLogin: false,
                });
        });
    }
    runIntegration(wm) {
        // We need menu on macOS, otherwise there are no shortcuts defined even for things like copy/paste or hiding window
        // this needs to be registered here because it's called after the app ready event
        const template = [
            {
                // Skip individual definitions because appMenu can do it automatically
                role: "appMenu",
            },
            {
                label: "Edit",
                submenu: [
                    {
                        role: "undo",
                    },
                    {
                        role: "redo",
                    },
                    {
                        type: "separator",
                    },
                    {
                        role: "cut",
                    },
                    {
                        role: "copy",
                    },
                    {
                        role: "paste",
                    },
                    {
                        role: "pasteAndMatchStyle",
                    },
                    {
                        role: "delete",
                    },
                    {
                        role: "selectAll",
                    },
                    {
                        type: "separator",
                    },
                    {
                        label: "Speech",
                        submenu: [
                            {
                                role: "startSpeaking",
                            },
                            {
                                role: "stopSpeaking",
                            },
                        ],
                    },
                ],
            },
            {
                label: "View",
                submenu: [
                    {
                        role: "togglefullscreen",
                    },
                ],
            },
            {
                role: "window",
                submenu: [
                    {
                        role: "close",
                    },
                    {
                        role: "minimize",
                    },
                    {
                        role: "zoom",
                    },
                    {
                        type: "separator",
                    },
                    {
                        role: "front",
                    },
                    {
                        role: "zoomIn",
                    },
                    {
                        role: "zoomOut",
                    },
                    {
                        role: "resetZoom",
                    },
                    {
                        click: () => {
                            wm.newWindow(true);
                        },
                        label: lang.get("openNewWindow_action"),
                        accelerator: "Command+N",
                        enabled: true,
                    },
                ],
            },
        ];
        const menu = this._electron.Menu.buildFromTemplate(template);
        this._electron.Menu.setApplicationMenu(menu);
        return Promise.resolve();
    }
    isIntegrated() {
        return Promise.resolve(true);
    }
    integrate() {
        return Promise.resolve();
    }
    unintegrate() {
        return Promise.resolve();
    }
}
//# sourceMappingURL=DesktopIntegratorDarwin.js.map