export class DesktopNativePushFacade {
    constructor(sse, alarmScheduler, alarmStorage) {
        this.sse = sse;
        this.alarmScheduler = alarmScheduler;
        this.alarmStorage = alarmStorage;
    }
    async closePushNotifications(addressesArray) {
        // only gets called in the app
        // the desktop client closes notifications on window focus
    }
    async getPushIdentifier() {
        const sseInfo = await this.sse.getSseInfo();
        return sseInfo?.identifier ?? null;
    }
    async initPushNotifications() {
        // Nothing to do here because sse connection is opened when starting the native part.
    }
    async scheduleAlarms(alarms) {
        for (const alarm of alarms) {
            await this.alarmScheduler.handleAlarmNotification(alarm);
        }
    }
    async storePushIdentifierLocally(identifier, userId, sseOrigin, pushIdentifierId, pushIdentifierSessionKey) {
        await this.sse.storePushIdentifier(identifier, userId, sseOrigin);
        await this.alarmStorage.storePushIdentifierSessionKey(pushIdentifierId, pushIdentifierSessionKey);
    }
    removeUser(userId) {
        return this.sse.removeUser(userId);
    }
    async invalidateAlarmsForUser(userId) {
        await this.alarmScheduler.unscheduleAllAlarms(userId);
    }
}
//# sourceMappingURL=DesktopNativePushFacade.js.map