import m from "mithril";
import { stateBgActive, stateBgHover } from "./builtinThemes.js";
import { theme } from "./theme.js";
import { styles } from "./styles.js";
import { px, size } from "./size.js";
import { DefaultAnimationTime } from "./animation/Animations.js";
export class SelectableRowContainer {
    constructor() {
        this.dom = null;
        this.selected = false;
        this.isInMultiselect = false;
        this.updateDomBg = () => {
            // in single column layout the "current element" selection is not meaningful and is even annoying
            const highlight = styles.isSingleColumnLayout() ? this.isInMultiselect && this.selected : this.selected;
            if (this.dom) {
                this.dom.style.backgroundColor = highlight ? stateBgHover : theme.list_bg;
            }
        };
    }
    view({ attrs, children }) {
        return m(".flex.mb-xs.border-radius.pt-m.pb-m.pl.pr.ml-s", {
            style: {
                paddingTop: "14px",
                paddingBottom: "12px",
                // this is an adjustment to keep tha columns aligned, space between columns is too big otherwise.
                // this is an obscure place to put it and ideally should not be done here or should be passed down here.
                marginRight: styles.isSingleColumnLayout() ? px(size.hpad_small) : "0",
                transition: `background 200ms`,
            },
            oncreate: ({ dom }) => {
                this.dom = dom;
                this.updateDomBg();
                attrs.onSelectedChangeRef?.((selected, isInMultiselect) => {
                    this.selected = selected;
                    this.isInMultiselect = isInMultiselect;
                    this.updateDomBg();
                });
            },
            onpointerdown: () => {
                if (this.dom)
                    this.dom.style.backgroundColor = stateBgActive;
            },
            onpointerup: this.updateDomBg,
            onpointercancel: this.updateDomBg,
            onpointerleave: this.updateDomBg,
        }, children);
    }
}
export function setVisibility(dom, visible) {
    dom.style.display = visible ? "" : "none";
}
export function checkboxOpacity(dom, selected) {
    if (selected) {
        dom.classList.remove("list-checkbox");
    }
    else {
        dom.classList.add("list-checkbox");
    }
}
export function shouldAlwaysShowMultiselectCheckbox() {
    return !styles.isUsingBottomNavigation();
}
// delay by 2 frames roughly so that the browser has time to do heavy stuff with layout
export const selectableRowAnimParams = { duration: DefaultAnimationTime, easing: "ease-in-out", fill: "forwards", delay: 36 };
export const scaleXHide = "scaleX(0)";
export const scaleXShow = "scaleX(1)";
//# sourceMappingURL=SelectableRowContainer.js.map