import m from "mithril";
import { lang } from "../../misc/LanguageViewModel";
import { getContentButtonIconBackground, getElevatedBackground, getNavButtonIconBackground, getNavigationMenuIcon, theme } from "../theme";
import { noOp } from "@tutao/tutanota-utils";
import { assertMainOrNode } from "../../api/common/Env";
import { BaseButton } from "./buttons/BaseButton.js";
assertMainOrNode();
export function getColors(buttonColors) {
    switch (buttonColors) {
        case "nav" /* ButtonColor.Nav */:
            return {
                button: theme.navigation_button,
                button_selected: theme.navigation_button_selected,
                button_icon_bg: getNavButtonIconBackground(),
                icon: theme.navigation_button_icon,
                icon_selected: theme.navigation_button_icon_selected,
                border: theme.navigation_bg,
            };
        case "drawernav" /* ButtonColor.DrawerNav */:
            return {
                button: theme.content_button,
                button_selected: theme.content_button_selected,
                button_icon_bg: "transparent",
                icon: getNavigationMenuIcon(),
                icon_selected: theme.content_button_icon_selected,
                border: getElevatedBackground(),
            };
        case "elevated" /* ButtonColor.Elevated */:
            return {
                button: theme.content_button,
                button_selected: theme.content_button_selected,
                button_icon_bg: getContentButtonIconBackground(),
                icon: theme.content_button_icon,
                icon_selected: theme.content_button_icon_selected,
                border: getElevatedBackground(),
            };
        case "content" /* ButtonColor.Content */:
        default:
            return {
                button: theme.content_button,
                button_selected: theme.content_button_selected,
                button_icon_bg: getContentButtonIconBackground(),
                icon: theme.content_button_icon,
                icon_selected: theme.content_button_icon_selected,
                border: theme.content_bg,
            };
    }
}
/**
 * A button.
 */
export class Button {
    view({ attrs }) {
        const getKey = lang.getMaybeLazy;
        const title = attrs.title == null ? getKey(attrs.label) : getKey(attrs.title);
        let classes = "limit-width noselect bg-transparent button-height text-ellipsis content-accent-fg flex items-center plr-button button-content justify-center flash";
        if (attrs.type === "primary" /* ButtonType.Primary */) {
            classes += " b";
        }
        return m(BaseButton, {
            label: title,
            text: getKey(attrs.label),
            class: classes,
            style: {
                borderColor: getColors(attrs.colors).border,
            },
            onclick: attrs.click ?? noOp,
        });
    }
}
//# sourceMappingURL=Button.js.map