import { SwipeHandler } from "./SwipeHandler.js";
import { animations, transform } from "../animation/Animations.js";
export class CalendarSwipeHandler extends SwipeHandler {
    constructor(touchArea, onGestureCompleted) {
        super(touchArea);
        this._xoffset = 0;
        // avoid flickering especially in day and week view when overflow-y is set on nested elements
        touchArea.style.transformStyle = "preserve-3d";
        touchArea.style.backfaceVisibility = "hidden";
        this._onGestureCompleted = onGestureCompleted;
    }
    onHorizontalDrag(xDelta, yDelta) {
        this._xoffset = Math.abs(xDelta) > 20 ? xDelta : 0;
        this.touchArea.style.transform = `translateX(${this._xoffset}px)`;
    }
    onHorizontalGestureCompleted(delta) {
        if (Math.abs(delta.x) > 100) {
            this._xoffset = 0;
            return animations
                .add(this.touchArea, transform("translateX" /* TransformEnum.TranslateX */, delta.x, this.touchArea.offsetWidth * (delta.x > 0 ? 1 : -1)))
                .then(() => {
                this._onGestureCompleted(delta.x < 0);
                requestAnimationFrame(() => {
                    this.touchArea.style.transform = "";
                });
            });
        }
        else {
            return this.reset(delta);
        }
    }
    reset(delta) {
        if (Math.abs(this._xoffset) > 20) {
            animations.add(this.touchArea, transform("translateX" /* TransformEnum.TranslateX */, delta.x, 0));
        }
        else {
            this.touchArea.style.transform = "";
        }
        this._xoffset = 0;
        return super.reset(delta);
    }
}
//# sourceMappingURL=CalendarSwipeHandler.js.map