import m from "mithril";
import { BootIconsSvg } from "./icons/BootIcons";
import { lang } from "../../misc/LanguageViewModel";
import { theme } from "../theme.js";
import { encodeSVG, getOperatingClasses } from "./GuiUtils.js";
export class Checkbox {
    constructor() {
        this.focused = false;
        this._domInput = null;
    }
    view(vnode) {
        const a = vnode.attrs;
        const helpLabelText = a.helpLabel ? lang.getMaybeLazy(a.helpLabel) : "";
        const helpLabel = a.helpLabel ? m(`small.block.content-fg${Checkbox.getBreakClass(helpLabelText)}`, helpLabelText) : [];
        return m(`.pt`, {
            role: "checkbox",
            "aria-checked": String(a.checked),
            "aria-disabled": String(a.disabled),
            class: getOperatingClasses(a.disabled, "click flash"),
            onclick: (e) => {
                if (e.target !== this._domInput) {
                    this.toggle(e, a); // event is bubbling in IE besides we invoke e.stopPropagation()
                }
            },
        }, m(`label${Checkbox.getBreakClass(a.label())}`, {
            class: `${this.focused ? "content-accent-fg" : "content-fg"} ${getOperatingClasses(a.disabled, "click")}`,
            onclick: (e) => {
                // if the label contains a link, then stop the event so that the checkbox doesn't get toggled upon clicking
                // we still allow it to be checked if they click on the non-link part of the label
                if (e.target instanceof HTMLElement && e.target.tagName.toUpperCase() === "A") {
                    e.stopPropagation();
                }
            },
        }, [
            m("input[type=checkbox].icon.checkbox-override", {
                oncreate: (vnode) => (this._domInput = vnode.dom),
                onchange: (e) => this.toggle(e, a),
                checked: a.checked,
                onfocus: () => (this.focused = true),
                onblur: () => (this.focused = false),
                class: getOperatingClasses(a.disabled, "click"),
                style: {
                    cursor: a.disabled ? "default" : "pointer",
                    "background-color": theme.content_accent,
                    "mask-image": `url("${a.checked ? Checkbox.checkedIcon : Checkbox.uncheckedIcon}")`,
                },
                disabled: a.disabled,
            }),
            a.label(),
            helpLabel,
        ]));
    }
    static getBreakClass(text) {
        if (typeof text !== "string" || text.includes(" ")) {
            return ".break-word";
        }
        else {
            return ".break-all";
        }
    }
    toggle(event, attrs) {
        if (!attrs.disabled) {
            attrs.onChecked(!attrs.checked);
        }
        event.stopPropagation();
        if (this._domInput) {
            this._domInput.focus();
        }
    }
}
Checkbox.checkedIcon = encodeSVG(BootIconsSvg["CheckboxSelected" /* BootIcons.CheckboxSelected */]);
Checkbox.uncheckedIcon = encodeSVG(BootIconsSvg["Checkbox" /* BootIcons.Checkbox */]);
//# sourceMappingURL=Checkbox.js.map