import m from "mithril";
import { NavButton } from "../base/NavButton.js";
import { size } from "../size";
import { CALENDAR_PREFIX, CONTACTS_PREFIX, MAIL_PREFIX, SEARCH_PREFIX } from "../../misc/RouteChange";
import { FeatureType } from "../../api/common/TutanotaConstants";
import { locator } from "../../api/main/MainLocator.js";
const fontSize = size.font_size_small;
function getHrefForSearch() {
    const route = m.route.get();
    if (route.startsWith(SEARCH_PREFIX)) {
        return route;
    }
    else if (route.startsWith(CONTACTS_PREFIX)) {
        return "/search/contact";
    }
    else if (route.startsWith(CALENDAR_PREFIX)) {
        return "/search/calendar";
    }
    else {
        return "/search/mail";
    }
}
export class BottomNav {
    view(vnode) {
        // Using bottom-nav class too to match it inside media queries like @print, otherwise it's not matched
        return m("nav.bottom-nav.flex.items-center.z1", [
            m(NavButton, {
                label: "emails_label",
                icon: () => "Mail" /* BootIcons.Mail */,
                href: MAIL_PREFIX,
                vertical: true,
                fontSize,
            }),
            locator.logins.isInternalUserLoggedIn()
                ? m(NavButton, {
                    label: "search_label",
                    icon: () => "Search" /* BootIcons.Search */,
                    href: () => getHrefForSearch(),
                    isSelectedPrefix: SEARCH_PREFIX,
                    vertical: true,
                    fontSize,
                })
                : null,
            locator.logins.isInternalUserLoggedIn() && !locator.logins.isEnabled(FeatureType.DisableContacts)
                ? m(NavButton, {
                    label: "contacts_label",
                    icon: () => "Contacts" /* BootIcons.Contacts */,
                    href: () => CONTACTS_PREFIX,
                    isSelectedPrefix: CONTACTS_PREFIX,
                    vertical: true,
                    fontSize,
                })
                : null,
            locator.logins.isInternalUserLoggedIn() && !locator.logins.isEnabled(FeatureType.DisableCalendar)
                ? m(NavButton, {
                    label: "calendar_label",
                    icon: () => "Calendar" /* BootIcons.Calendar */,
                    href: CALENDAR_PREFIX,
                    vertical: true,
                    fontSize,
                })
                : null,
        ]);
    }
}
//# sourceMappingURL=BottomNav.js.map