import { deviceConfig } from "../misc/DeviceConfig";
import { assertMainOrNodeBoot, isApp, isDesktop, isTest } from "../api/common/Env";
import { NativeThemeFacade, ThemeController, WebThemeFacade } from "./ThemeController";
import { isColorLight } from "./base/Color";
import { logo_text_bright_grey, logo_text_dark_grey } from "./builtinThemes";
import { getLogoSvg } from "./base/Logo";
assertMainOrNodeBoot();
const selectedThemeFacade = isApp() || isDesktop() ? new NativeThemeFacade() : new WebThemeFacade(deviceConfig);
// We need it because we want to run tests in node and real HTMLSanitizer does not work there.
const sanitizerStub = {
    sanitizeHTML: () => {
        return {
            html: "",
            blockedExternalContent: 0,
            inlineImageCids: [],
            links: [],
        };
    },
    sanitizeSVG(svg, configExtra) {
        throw new Error("stub!");
    },
    sanitizeFragment(html, configExtra) {
        throw new Error("stub!");
    },
};
const themeSingleton = {};
const lazySanitizer = isTest()
    ? () => Promise.resolve(sanitizerStub)
    : () => import("../misc/HtmlSanitizer").then(({ htmlSanitizer }) => htmlSanitizer);
export const themeController = new ThemeController(themeSingleton, selectedThemeFacade, lazySanitizer);
// For native targets WebCommonNativeFacade notifies themeController because Android and Desktop do not seem to work reliably via media queries
if (selectedThemeFacade instanceof WebThemeFacade) {
    selectedThemeFacade.addDarkListener(() => themeController.reloadTheme());
}
// ThemeController.updateTheme updates the object in place, so this will always be current.
// There are few alternative ways this could have been implemented:
//  * make each property on this singleton a getter that defers to themeController
//  * make this singleton a proxy that does the same thing
// We keep this singleton available because it is convenient to refer to, and already everywhere in the code before the addition of ThemeController.
export const theme = themeSingleton;
export const themeOptions = [
    {
        name: "systemThemePref_label",
        value: "auto:light|dark",
    },
    {
        name: "light_label",
        value: "light",
    },
    {
        name: "dark_label",
        value: "dark",
    },
    {
        name: "blue_label",
        value: "blue",
    },
];
export function getContentButtonIconBackground() {
    return theme.content_button_icon_bg || theme.content_button; // fallback for the new color content_button_icon_bg
}
export function getNavButtonIconBackground() {
    return theme.navigation_button_icon_bg || theme.navigation_button; // fallback for the new color content_button_icon_bg
}
export function getElevatedBackground() {
    return theme.elevated_bg || theme.content_bg;
}
export function getNavigationMenuBg() {
    return theme.navigation_menu_bg || theme.navigation_bg;
}
export function getNavigationMenuIcon() {
    return theme.navigation_menu_icon || theme.navigation_button_icon;
}
export function getColouredTutanotaLogo() {
    return getLogoSvg(theme.content_accent, isColorLight(theme.content_bg) ? logo_text_dark_grey : logo_text_bright_grey);
}
//# sourceMappingURL=theme.js.map