import m from "mithril";
import { TemplateGroupRootTypeRef } from "../../api/entities/tutanota/TypeRefs.js";
import { memoized, neverNull, noOp, ofClass, startsWith } from "@tutao/tutanota-utils";
import { htmlSanitizer } from "../../misc/HtmlSanitizer";
import { locator } from "../../api/main/MainLocator";
import { getConfirmation } from "../../gui/base/GuiUtils";
import { NotFoundError } from "../../api/common/error/RestError";
import { IconButton } from "../../gui/base/IconButton.js";
/**
 *  Renders one knowledgeBase entry
 */
export class KnowledgeBaseEntryView {
    constructor() {
        this._sanitizedEntry = memoized((entry) => {
            return {
                content: htmlSanitizer.sanitizeHTML(entry.description, {
                    blockExternalContent: true,
                }).html,
            };
        });
    }
    view({ attrs }) {
        return m(".flex.flex-column", [this._renderContent(attrs)]);
    }
    _renderContent(attrs) {
        const { entry, readonly } = attrs;
        return m("", {
            onclick: (event) => {
                this._handleAnchorClick(event, attrs);
            },
        }, [
            m(".flex.mt-l.center-vertically.selectable", m(".h4.text-ellipsis", entry.title), !readonly ? [m(".flex.flex-grow.justify-end", [this.renderEditButton(entry), this.renderRemoveButton(entry)])] : null),
            m("", [
                m(".mt-s.flex.mt-s.wrap", [
                    entry.keywords.map((entryKeyword) => {
                        return m(".keyword-bubble.selectable", entryKeyword.keyword);
                    }),
                ]),
                m(".flex.flex-column.mt-s", [m(".editor-border.text-break.selectable", m.trust(this._sanitizedEntry(entry).content))]),
            ]),
        ]);
    }
    renderRemoveButton(entry) {
        return m(IconButton, {
            title: "remove_action",
            icon: "Trash" /* Icons.Trash */,
            click: () => {
                getConfirmation("deleteEntryConfirm_msg").confirmed(() => locator.entityClient.erase(entry).catch(ofClass(NotFoundError, noOp)));
            },
        });
    }
    renderEditButton(entry) {
        return m(IconButton, {
            title: "edit_action",
            icon: "Edit" /* Icons.Edit */,
            click: () => {
                import("../../settings/KnowledgeBaseEditor").then(({ showKnowledgeBaseEditor }) => {
                    locator.entityClient.load(TemplateGroupRootTypeRef, neverNull(entry._ownerGroup)).then((groupRoot) => {
                        showKnowledgeBaseEditor(entry, groupRoot);
                    });
                });
            },
        });
    }
    _handleAnchorClick(event, attrs) {
        let target = event.target;
        if (target && target.closest) {
            let anchorElement = target.closest("a");
            if (anchorElement && startsWith(anchorElement.href, "tutatemplate:")) {
                event.preventDefault();
                const [listId, elementId] = new URL(anchorElement.href).pathname.split("/");
                attrs.onTemplateSelected([listId, elementId]);
            }
        }
    }
}
//# sourceMappingURL=KnowledgeBaseEntryView.js.map