import { ofClass } from "@tutao/tutanota-utils";
import { PermissionError } from "../api/common/error/PermissionError.js";
export class NoopCredentialRemovalHandler {
    async onCredentialsRemoved(credentialsAndDbKey) { }
}
export class AppsCredentialRemovalHandler {
    constructor(indexer, pushApp, configFacade, mobileContactsFacade) {
        this.indexer = indexer;
        this.pushApp = pushApp;
        this.configFacade = configFacade;
        this.mobileContactsFacade = mobileContactsFacade;
    }
    async onCredentialsRemoved(credentialsAndDbKey) {
        if (credentialsAndDbKey.databaseKey != null) {
            const { userId } = credentialsAndDbKey.credentials;
            await this.indexer.deleteIndex(userId);
            await this.pushApp.invalidateAlarmsForUser(userId);
            await this.pushApp.removeUserFromNotifications(userId);
            await this.configFacade.delete(userId);
        }
        await this.mobileContactsFacade
            ?.deleteContacts(credentialsAndDbKey.credentials.login, null)
            .catch(ofClass(PermissionError, (e) => console.log("No permission to clear contacts", e)));
    }
}
//# sourceMappingURL=CredentialRemovalHandler.js.map