import m from "mithril";
import { LoginScreenHeader } from "../gui/LoginScreenHeader.js";
import { styles } from "../gui/styles.js";
import { landmarkAttrs } from "../gui/AriaUtils.js";
import { lang } from "../misc/LanguageViewModel.js";
import { renderInfoLinks } from "./LoginView.js";
import { BaseTopLevelView } from "../gui/BaseTopLevelView.js";
import { Table } from "../gui/base/Table.js";
import { LoginButton } from "../gui/base/buttons/LoginButton.js";
export const TAG = "[CredentialsMigrationView]";
/**
 * there's two ways we can get here:
 * * the old domain opened the new domain as a new tab on this view. we want to send a "ready" event and then receive credentials, then send "done" event.
 * * the new domain opened the old domain as a new tab on this view. we want to send credentials, then prompt the user to close this tab.
 * */
export class CredentialsMigrationView extends BaseTopLevelView {
    oncreate(vnode) {
        this.viewModel = vnode.attrs.credentialsMigrationViewModel;
        window.addEventListener("beforeunload", () => this.viewModel.cancelMigration());
    }
    onNewUrl(args, requestedPath) {
        // ignore
    }
    constructor({ attrs }) {
        super();
        this.viewModel = attrs.credentialsMigrationViewModel;
    }
    view(vnode) {
        if (window.opener == null) {
            // user navigated here through some link or so, not by our intended mechanism.
            m.route.set("/login");
            return null;
        }
        return m("#login-view.main-view.flex.col.nav-bg", [
            m(LoginScreenHeader),
            m(".flex-grow.flex-center.scroll", m(".flex.col.flex-grow-shrink-auto.max-width-m.plr-l." + (styles.isSingleColumnLayout() ? "pt" : "pt-l"), {
                ...landmarkAttrs("main" /* AriaLandmarks.Main */, lang.get("login_label")),
                oncreate: (vnode) => {
                    ;
                    vnode.dom.focus();
                },
            }, [
                m(".content-bg.border-radius-big", {
                    class: styles.isSingleColumnLayout() ? "plr-l" : "plr-2l",
                }, m(".flex.col.pb-l.pt-l", this.renderMigrationForm())),
                m(".flex-grow"),
                renderInfoLinks(),
            ])),
        ]);
    }
    renderMigrationForm() {
        switch (this.viewModel.migrationState) {
            case 0 /* MigrationState.GettingCredentials */:
                return this.renderSpinner();
            case 1 /* MigrationState.WaitingForInput */:
                return this.renderTransferPrompt();
            case 2 /* MigrationState.Transferring */:
                return this.renderSpinner();
            case 3 /* MigrationState.Complete */:
                return this.renderCompleteMessage();
        }
    }
    renderSpinner() {
        return m(".text-center", lang.get("pleaseWait_msg"));
    }
    renderTransferPrompt() {
        const savedCredentials = this.viewModel.logins.getSavedCredentials();
        if (savedCredentials.length === 0) {
            this.viewModel.executeMigration();
            return this.renderSpinner();
        }
        else {
            return [
                lang.get("transferCredentials_msg"),
                m(".mt.mb", m(Table, {
                    columnHeading: ["knownCredentials_label"],
                    showActionButtonColumn: false,
                    columnWidths: [".column-width-largest" /* ColumnWidth.Largest */],
                    lines: savedCredentials.map((c) => ({ cells: [c.login] })),
                })),
                m(LoginButton, {
                    label: "apply_action",
                    onclick: () => this.viewModel.executeMigration(),
                }),
            ];
        }
    }
    renderCompleteMessage() {
        return m(".text-center", [
            m(".mb", lang.get("credentialMigrationDone_msg")),
            m(LoginButton, {
                label: "close_alt",
                onclick: () => window.close(),
            }),
        ]);
    }
}
//# sourceMappingURL=CredentialsMigrationView.js.map