import { MessageDispatcher, Request } from "../api/common/threading/MessageDispatcher.js";
import { CrossOriginTransport } from "../api/common/threading/CrossOriginTransport.js";
import { isLegacyDomain } from "./LoginViewModel.js";
import m from "mithril";
/** should only ever be used in contexts where we were opened as a new tab from the legacy domain and we're the new domain, or
 * as a new tab from the new domain and we're the old domain.*/
export class CredentialsMigrationViewModel {
    constructor(logins, parentOrigin) {
        this.logins = logins;
        this.parentOrigin = parentOrigin;
        this.aborted = false;
        this.migrationState = 0 /* MigrationState.GettingCredentials */;
        this.transport = null;
        this.dispatcher = null;
        this.initPromise = this.logins.init().then(() => {
            this.migrationState = 1 /* MigrationState.WaitingForInput */;
            m.redraw();
        });
        logins.init().then(() => (this.migrationState = 1 /* MigrationState.WaitingForInput */));
        if (isLegacyDomain()) {
        }
        else {
        }
    }
    async executeMigration() {
        this.migrationState = 2 /* MigrationState.Transferring */;
        console.log("attempting credentials migration");
        if (this.aborted)
            return;
        try {
            if (isLegacyDomain()) {
                await this.executeMigrationAsOldDomain();
            }
            else {
                await this.executeMigrationAsNewDomain();
            }
        }
        catch (e) {
            if (this.aborted)
                return;
            console.log("error during migration", e);
        }
        finally {
            this.migrationState = 3 /* MigrationState.Complete */;
        }
    }
    /** notify the other side that this tab is going away in some way. */
    async cancelMigration() {
        this.dispatcher?.postRequest(new Request("abort", []));
        this.transport?.dispose();
    }
    async executeMigrationAsOldDomain() {
        this.transport = new CrossOriginTransport(window.opener, this.parentOrigin);
        const commands = {
            abort: async () => (this.aborted = true),
            ready: async () => {
                await this.dispatcher?.postRequest(new Request("credentials", [this.logins.getAllCredentials()]));
                if (this.aborted)
                    return;
                await this.logins.deleteAllCredentials();
            },
        };
        this.dispatcher = new MessageDispatcher(this.transport, commands, "tab-main");
        // not doing this through the dispatcher, the other side is not listening yet.
        window.opener.postMessage("init", this.parentOrigin);
    }
    async executeMigrationAsNewDomain() {
        console.log(this.parentOrigin);
        this.transport = new CrossOriginTransport(window.opener, this.parentOrigin);
        const commands = {
            credentials: async (req) => {
                await this.logins.addAllCredentials(req.args[0]);
                await this.dispatcher?.postRequest(new Request("done", []));
                this.transport?.dispose();
                m.route.set("/login");
            },
        };
        this.dispatcher = new MessageDispatcher(this.transport, commands, "tab-main");
        window.opener.postMessage("init", this.parentOrigin);
    }
}
//# sourceMappingURL=CredentialsMigrationViewModel.js.map