import m from "mithril";
import stream from "mithril/stream";
import { liveDataAttrs } from "../gui/AriaUtils";
import { lang } from "../misc/LanguageViewModel";
import { TextField } from "../gui/base/TextField.js";
import { Checkbox } from "../gui/base/Checkbox.js";
import { client } from "../misc/ClientDetector";
import { isApp, isDesktop, isOfflineStorageAvailable } from "../api/common/Env";
import { getWhitelabelCustomizations } from "../misc/WhitelabelCustomizations.js";
import { ACTIVATED_MIGRATION, isLegacyDomain } from "./LoginViewModel.js";
import { LoginButton } from "../gui/base/buttons/LoginButton.js";
export class LoginForm {
    oncreate(vnode) {
        const a = vnode.attrs;
        this.autofillUpdateHandler = stream.combine(() => {
            requestAnimationFrame(() => {
                const oldAddress = a.mailAddress();
                const newAddress = this.mailAddressTextField.value;
                const oldPassword = a.password();
                const newPassword = this.passwordTextField.value;
                // only update values when they are different or we get stuck in an infinite loop
                if (oldAddress !== newAddress && newAddress != "")
                    a.mailAddress(newAddress);
                if (oldPassword !== newPassword && newPassword != "")
                    a.password(newPassword);
            });
        }, [a.mailAddress, a.password]);
    }
    onremove(vnode) {
        vnode.attrs.password("");
        this.autofillUpdateHandler.end(true);
        this.passwordTextField.value = "";
    }
    _passwordDisabled() {
        // some whitelabel customers have disabled password saving
        const hasCustomDisabled = getWhitelabelCustomizations(window)?.bootstrapCustomizations?.includes("0" /* BootstrapFeatureType.DisableSavePassword */) != null;
        // on the old domain, we don't want to save new credentials.
        const noSaveLegacyDomain = ACTIVATED_MIGRATION() && isLegacyDomain();
        return hasCustomDisabled || noSaveLegacyDomain;
    }
    view(vnode) {
        const a = vnode.attrs;
        const canSaveCredentials = client.localStorage();
        if (a.savePassword && (isApp() || isDesktop()) && !this._passwordDisabled()) {
            a.savePassword(true);
        }
        return m("form", {
            onsubmit: (e) => {
                // do not post the form, the form is just here to enable browser auto-fill
                e.preventDefault(); // a.onSubmit(a.mailAddress(), a.password())
            },
        }, [
            m("", m(TextField, {
                label: "mailAddress_label",
                value: a.mailAddress(),
                oninput: a.mailAddress,
                type: "email" /* TextFieldType.Email */,
                autocompleteAs: "email" /* Autocomplete.email */,
                onDomInputCreated: (dom) => {
                    this.mailAddressTextField = dom;
                    if (!client.isMobileDevice()) {
                        dom.focus(); // have email address auto-focus so the user can immediately type their username (unless on mobile)
                    }
                },
            })),
            m("", m(TextField, {
                label: "password_label",
                value: a.password(),
                oninput: a.password,
                type: "password" /* TextFieldType.Password */,
                autocompleteAs: "current-password" /* Autocomplete.currentPassword */,
                onDomInputCreated: (dom) => (this.passwordTextField = dom),
            })),
            a.savePassword && !this._passwordDisabled()
                ? isApp() || isDesktop()
                    ? m("small.block.content-fg", lang.get("dataWillBeStored_msg"))
                    : m(Checkbox, {
                        label: () => lang.get("storePassword_action"),
                        checked: a.savePassword(),
                        onChecked: a.savePassword,
                        helpLabel: canSaveCredentials
                            ? () => lang.get("onlyPrivateComputer_msg") + (isOfflineStorageAvailable() ? "\n" + lang.get("dataWillBeStored_msg") : "")
                            : "functionNotSupported_msg",
                        disabled: !canSaveCredentials,
                    })
                : null,
            m(".pt", m(LoginButton, {
                label: isApp() || isDesktop() ? "addAccount_action" : "login_action",
                onclick: () => a.onSubmit(a.mailAddress(), a.password()),
            })),
            m("p.center.statusTextColor.mt-s", {
                style: {
                    // browser reset
                    marginBottom: 0,
                },
            }, m("small", liveDataAttrs(), [
                a.helpText ? a.helpText : null,
                " ",
                a.invalidCredentials && a.showRecoveryOption
                    ? m("a", {
                        href: "/recover",
                        onclick: (e) => {
                            m.route.set("/recover", {
                                mailAddress: a.mailAddress(),
                                resetAction: "password",
                            });
                            e.preventDefault();
                        },
                    }, lang.get("recoverAccountAccess_action"))
                    : a.accessExpired && a.accessExpired
                        ? m("a", {
                            // We import the dialog directly rather than redirecting to /recover here in order to not pass the password in plaintext via the URL
                            href: "#",
                            onclick: (e) => {
                                import("./recover/TakeOverDeletedAddressDialog").then(({ showTakeOverDialog }) => showTakeOverDialog(a.mailAddress(), a.password()));
                                e.preventDefault();
                            },
                        }, lang.get("help_label"))
                        : null,
            ])),
        ]);
    }
}
//# sourceMappingURL=LoginForm.js.map