import { FileTypeRef } from "../../api/entities/tutanota/TypeRefs.js";
import { getLetId } from "../../api/common/utils/EntityUtils";
import { promiseMap } from "@tutao/tutanota-utils";
import { loadMailDetails, loadMailHeaders } from "../model/MailUtils.js";
import { getDisplayedSender } from "../../api/common/mail/CommonMailUtils.js";
/**
 * Downloads the mail body and the attachments for an email, to prepare for exporting
 */
export async function makeMailBundle(mail, mailFacade, entityClient, fileController, sanitizer, cryptoFacade) {
    const mailWrapper = await loadMailDetails(mailFacade, entityClient, mail);
    const body = sanitizer.sanitizeHTML(mailWrapper.getMailBodyText(), {
        blockExternalContent: false,
        allowRelativeLinks: false,
        usePlaceholderForInlineImages: false,
    }).html;
    const files = await promiseMap(mail.attachments, async (fileId) => await entityClient.load(FileTypeRef, fileId));
    const attachments = await promiseMap(await cryptoFacade.enforceSessionKeyUpdateIfNeeded(mail, files), async (file) => await fileController.getAsDataFile(file));
    const headers = await loadMailHeaders(entityClient, mailWrapper);
    const recipientMapper = ({ address, name }) => ({ address, name });
    return {
        mailId: getLetId(mail),
        subject: mail.subject,
        body,
        sender: recipientMapper(getDisplayedSender(mail)),
        to: mailWrapper.getToRecipients().map(recipientMapper),
        cc: mailWrapper.getCcRecipients().map(recipientMapper),
        bcc: mailWrapper.getBccRecipients().map(recipientMapper),
        replyTo: mailWrapper.getReplyTos().map(recipientMapper),
        isDraft: mail.state === "0" /* MailState.DRAFT */,
        isRead: !mail.unread,
        sentOn: mailWrapper.getSentDate().getTime(),
        receivedOn: mail.receivedDate.getTime(),
        headers,
        attachments,
    };
}
//# sourceMappingURL=Bundler.js.map