/**
 * Platform-independent implementation for of ICredentialsKeyMigrator.
 */
export class DefaultCredentialsKeyMigrator {
    constructor(nativeCredentials) {
        this.nativeCredentials = nativeCredentials;
    }
    async migrateCredentialsKey(oldKeyEncrypted, oldMode, newMode) {
        const oldKeyDecryptedKeyBase64 = await this.nativeCredentials.decryptUsingKeychain(oldKeyEncrypted, oldMode);
        return await this.nativeCredentials.encryptUsingKeychain(oldKeyDecryptedKeyBase64, newMode);
    }
}
/**
 * Stub to be used on platforms for which we have not implemented credentials encryption with an intermediate key yet.
 * It will throw an error when called, since no key migration should ever be triggered on platforms that don't support the feature.
 */
export class StubCredentialsKeyMigrator {
    migrateCredentialsKey(oldKeyEncrypted, oldMode, newMode) {
        throw new Error("Should not be used");
    }
}
//# sourceMappingURL=CredentialsKeyMigrator.js.map