import { isAndroidApp } from "../../api/common/Env.js";
import { PermissionError } from "../../api/common/error/PermissionError.js";
import { Dialog } from "../../gui/base/Dialog.js";
export class SystemPermissionHandler {
    constructor(systemFacade) {
        this.systemFacade = systemFacade;
    }
    async queryPermissionsState() {
        return {
            isNotificationPermissionGranted: await this.hasPermission("2" /* PermissionType.Notification */),
            isBatteryPermissionGranted: isAndroidApp() ? await this.hasPermission("1" /* PermissionType.IgnoreBatteryOptimization */) : true,
        };
    }
    async hasPermission(permission) {
        return await this.systemFacade.hasPermission(permission);
    }
    async requestPermission(permission, deniedMessage) {
        try {
            await this.systemFacade.requestPermission(permission);
            return true;
        }
        catch (e) {
            if (e instanceof PermissionError) {
                console.warn("Permission denied for", permission);
                Dialog.confirm(deniedMessage).then((confirmed) => {
                    if (confirmed) {
                        this.systemFacade.goToSettings();
                    }
                });
                return false;
            }
            throw e;
        }
    }
}
//# sourceMappingURL=SystemPermissionHandler.js.map