import { createWizardDialog, wizardPageWrapper } from "../../../gui/base/WizardDialog.js";
import { defer } from "@tutao/tutanota-utils";
import { SetupCongratulationsPage, SetupCongratulationsPageAttrs } from "./setupwizardpages/SetupCongraulationsPage.js";
import m from "mithril";
import { SetupNotificationsPage, SetupNotificationsPageAttrs } from "./setupwizardpages/SetupNotificationsPage.js";
import { BannerButton } from "../../../gui/base/buttons/BannerButton.js";
import { theme } from "../../../gui/theme.js";
import { SetupThemePage, SetupThemePageAttrs } from "./setupwizardpages/SetupThemePage.js";
import { SetupContactsPage, SetupContactsPageAttrs } from "./setupwizardpages/SetupContactsPage.js";
import { SetupLockPage, SetupLockPageAttrs } from "./setupwizardpages/SetupLockPage.js";
export function renderPermissionButton(permissionName, isPermissionGranted, onclick) {
    return renderBannerButton(isPermissionGranted ? "granted_msg" : permissionName, onclick, isPermissionGranted);
}
export function renderBannerButton(text, onclick, isDisabled, classes) {
    return m(BannerButton, {
        text,
        borderColor: theme.content_accent,
        color: theme.content_accent,
        class: "b full-width button-content " + classes,
        click: (event, dom) => {
            onclick(event, dom);
        },
        disabled: isDisabled ?? undefined,
    });
}
export async function showSetupWizard(systemPermissionHandler, webMobileFacade, contactImporter, systemFacade, credentialsProvider, contactSyncManager, deviceConfig) {
    const wizardPages = [
        wizardPageWrapper(SetupCongratulationsPage, new SetupCongratulationsPageAttrs()),
        wizardPageWrapper(SetupNotificationsPage, new SetupNotificationsPageAttrs(await systemPermissionHandler.queryPermissionsState(), webMobileFacade.getIsAppVisible(), systemPermissionHandler)),
        wizardPageWrapper(SetupThemePage, new SetupThemePageAttrs()),
        wizardPageWrapper(SetupContactsPage, new SetupContactsPageAttrs(contactSyncManager, contactImporter, systemFacade)),
        wizardPageWrapper(SetupLockPage, new SetupLockPageAttrs(credentialsProvider)),
    ];
    const deferred = defer();
    const wizardBuilder = createWizardDialog(null, wizardPages, async () => {
        deviceConfig.setIsSetupComplete(true);
        deferred.resolve();
    }, "EditSmall" /* DialogType.EditSmall */);
    wizardBuilder.dialog.show();
    return deferred.promise;
}
//# sourceMappingURL=SetupWizard.js.map