import m from "mithril";
import { lang } from "../../../../misc/LanguageViewModel.js";
import { SetupPageLayout } from "./SetupPageLayout.js";
import { Dialog } from "../../../../gui/base/Dialog.js";
import { renderBannerButton } from "../SetupWizard.js";
export class SetupContactsPage {
    view({ attrs }) {
        const isContactSyncEnabled = attrs.syncManager.isEnabled();
        return m(SetupPageLayout, { image: "contacts" }, [
            m("p.mb-s", lang.get("importContacts_msg")),
            renderBannerButton("import_action", () => {
                attrs.contactImporter.importContactsFromDeviceSafely();
            }),
            m("p.mb-s", lang.get("allowContactSynchronization")),
            renderBannerButton(isContactSyncEnabled ? "activated_label" : "activate_action", () => {
                this.enableSync(attrs);
            }, isContactSyncEnabled, "mb-l"),
        ]);
    }
    async enableSync(attrs) {
        attrs.syncManager.enableSync();
        const success = await attrs.syncManager.syncContacts();
        if (!success) {
            await attrs.syncManager.disableSync();
            await Dialog.message("allowContactReadWrite_msg");
            await attrs.mobileSystemFacade.goToSettings();
        }
    }
}
export class SetupContactsPageAttrs {
    constructor(syncManager, contactImporter, mobileSystemFacade) {
        this.syncManager = syncManager;
        this.contactImporter = contactImporter;
        this.mobileSystemFacade = mobileSystemFacade;
        this.hidePagingButtonForPage = false;
        this.data = null;
    }
    headerTitle() {
        return lang.get("contacts_label");
    }
    nextAction(showDialogs) {
        // next action not available for this page
        return Promise.resolve(true);
    }
    isSkipAvailable() {
        return false;
    }
    isEnabled() {
        return true;
    }
}
//# sourceMappingURL=SetupContactsPage.js.map