import m from "mithril";
import { isAndroidApp } from "../../../../api/common/Env.js";
import { lang } from "../../../../misc/LanguageViewModel.js";
import { renderPermissionButton } from "../SetupWizard.js";
import { SetupPageLayout } from "./SetupPageLayout.js";
import { locator } from "../../../../api/main/MainLocator.js";
export class SetupNotificationsPage {
    view({ attrs }) {
        return m(SetupPageLayout, { image: "notifications" }, [
            m("p.mb-s", lang.get("allowNotifications_msg")),
            renderPermissionButton("grant_notification_permission_action", attrs.data.isNotificationPermissionGranted, () => attrs.askForNotificationPermission()),
            !isAndroidApp()
                ? null
                : m("section.mt-s.mb", [
                    m("p.mb-s.mt-s", lang.get("allowBatteryPermission_msg")),
                    renderPermissionButton("grant_battery_permission_action", attrs.data.isBatteryPermissionGranted, () => attrs.askForBatteryNotificationPermission()),
                ]),
        ]);
    }
}
export class SetupNotificationsPageAttrs {
    constructor(permissionData, visiblityStream, systemPermissionHandler) {
        this.systemPermissionHandler = systemPermissionHandler;
        this.hidePagingButtonForPage = false;
        this.isPageVisible = this.isPageNeeded(permissionData);
        this.data = permissionData;
        visiblityStream.map((isVisible) => {
            // Redraw the page when the user resumes the app to check for changes in permissions
            if (isVisible) {
                this.systemPermissionHandler.queryPermissionsState().then((permissionState) => {
                    this.data = permissionState;
                    m.redraw();
                });
            }
        });
    }
    async askForNotificationPermission() {
        // Ask for the notification permission
        const isNotificationPermissionGranted = await this.systemPermissionHandler.requestPermission("2" /* PermissionType.Notification */, "grant_notification_permission_action");
        this.data = {
            ...this.data,
            isNotificationPermissionGranted,
        };
        // Register the push notifications if granted
        if (isNotificationPermissionGranted) {
            locator.pushService.register();
        }
        m.redraw();
    }
    async askForBatteryNotificationPermission() {
        // Ask for permission to disable battery optimisations
        this.data = {
            ...this.data,
            isBatteryPermissionGranted: await this.systemPermissionHandler.requestPermission("1" /* PermissionType.IgnoreBatteryOptimization */, "allowBatteryPermission_msg"),
        };
        m.redraw();
    }
    isPageNeeded(data) {
        // Skip this page if the needed permissions are already granted
        if (isAndroidApp()) {
            return !data.isNotificationPermissionGranted || !data.isBatteryPermissionGranted;
        }
        return !data.isNotificationPermissionGranted;
    }
    headerTitle() {
        return lang.get("notificationSettings_action");
    }
    nextAction(showDialogs) {
        // next action not available for this page
        return Promise.resolve(true);
    }
    isSkipAvailable() {
        return false;
    }
    isEnabled() {
        return this.isPageVisible;
    }
}
//# sourceMappingURL=SetupNotificationsPage.js.map