import m from "mithril";
import { lang } from "../../../../misc/LanguageViewModel.js";
import { RadioSelector } from "../../../../gui/base/RadioSelector.js";
import { themeController, themeOptions } from "../../../../gui/theme.js";
import { SetupPageLayout } from "./SetupPageLayout.js";
export class SetupThemePage {
    constructor() {
        // The whitelabel themes formatted as `RadioSelectorOption`s.
        this.customThemes = null;
    }
    oninit() {
        // Get the whitelabel themes from the theme controller and map them to `RadioSelector` options.
        themeController.getCustomThemes().then((customThemes) => {
            this.customThemes = customThemes.map((themeId) => {
                return { name: () => themeId, value: themeId };
            });
            m.redraw();
        });
    }
    view() {
        return m(SetupPageLayout, {
            image: "theme",
        }, m("p.full-width", lang.get("theme_title")), 
        // We need to await the promise from `themeController.getCustomThemes()`, so we delay rendering the `RadioSelector` until it does.
        this.customThemes == null
            ? null
            : m(RadioSelector, {
                name: "theme_label",
                options: [...themeOptions, ...this.customThemes],
                class: "mb-s",
                selectedOption: themeController.themePreference,
                onOptionSelected: (option) => {
                    themeController.setThemePreference(option, true);
                },
            }));
    }
}
export class SetupThemePageAttrs {
    constructor() {
        this.hidePagingButtonForPage = false;
        this.data = null;
    }
    headerTitle() {
        return lang.get("appearanceSettings_label");
    }
    nextAction(showDialogs) {
        // next action not available for this page
        return Promise.resolve(true);
    }
    isSkipAvailable() {
        return false;
    }
    isEnabled() {
        return true;
    }
}
//# sourceMappingURL=SetupThemePage.js.map