import m from "mithril";
import { assertMainOrNode } from "../../api/common/Env";
import { downcast, isSameTypeRef } from "@tutao/tutanota-utils";
import { MailRow } from "../../mail/view/MailRow";
import { List } from "../../gui/base/List.js";
import { size } from "../../gui/size.js";
import { KindaContactRow } from "../../contacts/view/ContactListView.js";
import { CalendarEventTypeRef, ContactTypeRef } from "../../api/entities/tutanota/TypeRefs.js";
import ColumnEmptyMessageBox from "../../gui/base/ColumnEmptyMessageBox.js";
import { lang } from "../../misc/LanguageViewModel.js";
import { theme } from "../../gui/theme.js";
import { styles } from "../../gui/styles.js";
import { KindaCalendarRow } from "../../calendar/view/CalendarRow.js";
assertMainOrNode();
export class SearchResultListEntry {
    constructor(entry) {
        this.entry = entry;
    }
    get _id() {
        return this.entry._id;
    }
}
export class SearchListView {
    constructor({ attrs }) {
        this.calendarRenderConfig = {
            itemHeight: size.list_row_height,
            multiselectionAllowed: 0 /* MultiselectMode.Disabled */,
            swipe: null,
            createElement: (dom) => {
                const row = new SearchResultListRow(new KindaCalendarRow(dom));
                m.render(dom, row.render());
                return row;
            },
        };
        this.mailRenderConfig = {
            itemHeight: size.list_row_height,
            multiselectionAllowed: 1 /* MultiselectMode.Enabled */,
            swipe: null,
            createElement: (dom) => {
                const row = new SearchResultListRow(new MailRow(true, () => row.entity && this.listModel.onSingleExclusiveSelection(row.entity)));
                m.render(dom, row.render());
                return row;
            },
        };
        this.contactRenderConfig = {
            itemHeight: size.list_row_height,
            multiselectionAllowed: 1 /* MultiselectMode.Enabled */,
            swipe: null,
            createElement: (dom) => {
                const row = new SearchResultListRow(new KindaContactRow(dom, () => row.entity && this.listModel.onSingleExclusiveSelection(row.entity)));
                m.render(dom, row.render());
                return row;
            },
        };
        this.listModel = attrs.listModel;
    }
    view({ attrs }) {
        this.listModel = attrs.listModel;
        const { icon, renderConfig } = this.getRenderItems(attrs.currentType);
        return attrs.listModel.isEmptyAndDone()
            ? m(ColumnEmptyMessageBox, {
                icon,
                message: () => lang.get("searchNoResults_msg") + "\n" + (attrs.isFreeAccount ? lang.get("goPremium_msg") : lang.get("switchSearchInMenu_label")),
                color: theme.list_message_bg,
            })
            : m(List, {
                state: attrs.listModel.state,
                renderConfig,
                onLoadMore: () => {
                    attrs.listModel?.loadMore();
                },
                onRetryLoading: () => {
                    attrs.listModel?.retryLoading();
                },
                onSingleSelection: (item) => {
                    attrs.listModel?.onSingleSelection(item);
                    attrs.onSingleSelection(item);
                },
                onSingleTogglingMultiselection: (item) => {
                    attrs.listModel.onSingleInclusiveSelection(item, styles.isSingleColumnLayout());
                },
                onRangeSelectionTowards: (item) => {
                    attrs.listModel.selectRangeTowards(item);
                },
                onStopLoading() {
                    if (attrs.cancelCallback != null) {
                        attrs.cancelCallback();
                    }
                    attrs.listModel.stopLoading();
                },
            });
    }
    getRenderItems(type) {
        if (isSameTypeRef(type, ContactTypeRef)) {
            return {
                icon: "Contacts" /* BootIcons.Contacts */,
                renderConfig: this.contactRenderConfig,
            };
        }
        else if (isSameTypeRef(type, CalendarEventTypeRef)) {
            return {
                icon: "Calendar" /* BootIcons.Calendar */,
                renderConfig: this.calendarRenderConfig,
            };
        }
        else {
            return {
                icon: "Mail" /* BootIcons.Mail */,
                renderConfig: this.mailRenderConfig,
            };
        }
    }
}
export class SearchResultListRow {
    get entity() {
        return this._entity;
    }
    constructor(delegate) {
        // set from List
        this.domElement = null;
        // this is our own entry which we need for some reason (probably easier to deal with than a lot of sum type entries)
        this._entity = null;
        this._delegate = delegate;
        this.top = 0;
    }
    update(entry, selected, isInMultiSelect) {
        this._delegate.domElement = this.domElement;
        this._entity = entry;
        this._delegate.update(downcast(entry.entry), selected, isInMultiSelect);
    }
    render() {
        return this._delegate.render();
    }
}
//# sourceMappingURL=SearchListView.js.map