import m from "mithril";
import { GroupInfoTypeRef } from "../api/entities/sys/TypeRefs.js";
import { GroupType } from "../api/common/TutanotaConstants";
import { locator } from "../api/main/MainLocator";
import { noOp, promiseMap } from "@tutao/tutanota-utils";
import { isUpdateForTypeRef } from "../api/common/utils/EntityUpdateUtils.js";
class LocalAdminGroupInfoModel {
    constructor() {
        this._initialization = null;
        this.groupInfos = [];
    }
    init() {
        if (this._initialization) {
            return this._initialization;
        }
        locator.eventController.addEntityListener((updates) => {
            return promiseMap(updates, (update) => {
                return this.entityEventReceived(update);
            }).then(noOp);
        });
        return this._init();
    }
    _init() {
        this._initialization = locator.logins
            .getUserController()
            .loadCustomer()
            .then(async (customer) => {
            const groupInfos = await locator.entityClient.loadAll(GroupInfoTypeRef, customer.teamGroups);
            this.groupInfos = groupInfos.filter((gi) => gi.groupType === GroupType.LocalAdmin);
            return this.groupInfos;
        });
        return this._initialization;
    }
    entityEventReceived(update) {
        if (isUpdateForTypeRef(GroupInfoTypeRef, update)) {
            return this._init().then(() => m.redraw());
        }
        else {
            return Promise.resolve();
        }
    }
}
export const localAdminGroupInfoModel = new LocalAdminGroupInfoModel();
//# sourceMappingURL=LocalAdminGroupInfoModel.js.map