import m from "mithril";
import { Dialog } from "../gui/base/Dialog";
import { lang } from "../misc/LanguageViewModel";
import { InvoiceDataInput, InvoiceDataInputLocation } from "./InvoiceDataInput";
import { PaymentMethodInput } from "./PaymentMethodInput";
import stream from "mithril/stream";
import { getClientType, Keys, PaymentMethodTypeToName } from "../api/common/TutanotaConstants";
import { showProgressDialog } from "../gui/dialogs/ProgressDialog";
import { AccountingInfoTypeRef, InvoiceInfoTypeRef } from "../api/entities/sys/TypeRefs.js";
import { assertNotNull, neverNull, noOp, promiseMap } from "@tutao/tutanota-utils";
import { getLazyLoadedPayPalUrl, getPreconditionFailedPaymentMsg } from "./SubscriptionUtils";
import { Button } from "../gui/base/Button.js";
import { SegmentControl } from "../gui/base/SegmentControl";
import { emitWizardEvent } from "../gui/base/WizardDialog.js";
import { DefaultAnimationTime } from "../gui/animation/Animations";
import { locator } from "../api/main/MainLocator";
import { isUpdateForTypeRef } from "../api/common/utils/EntityUpdateUtils.js";
import { LoginButton } from "../gui/base/buttons/LoginButton.js";
/**
 * Wizard page for editing invoice and payment data.
 */
export class InvoiceAndPaymentDataPage {
    constructor(upgradeData) {
        this._paymentMethodInput = null;
        this._invoiceDataInput = null;
        this._availablePaymentMethods = null;
        this.__signupPaidTest = locator.usageTestController.getTest("signup.paid");
        this.__paymentPaypalTest = locator.usageTestController.getTest("payment.paypal");
        this._upgradeData = upgradeData;
        this._selectedPaymentMethod = stream();
        this._selectedPaymentMethod.map((method) => neverNull(this._paymentMethodInput).updatePaymentMethod(method));
    }
    onremove(vnode) {
        const data = vnode.attrs.data;
        // TODO check if correct place to update these
        if (this._invoiceDataInput && this._paymentMethodInput) {
            data.invoiceData = this._invoiceDataInput.getInvoiceData();
            data.paymentData = this._paymentMethodInput.getPaymentData();
        }
    }
    oncreate(vnode) {
        this.dom = vnode.dom;
        const data = vnode.attrs.data;
        // TODO check if correct place to update these
        if (this._invoiceDataInput && this._paymentMethodInput) {
            data.invoiceData = this._invoiceDataInput.getInvoiceData();
            data.paymentData = this._paymentMethodInput.getPaymentData();
        }
        let login = Promise.resolve(null);
        if (!locator.logins.isUserLoggedIn()) {
            login = locator.logins
                .createSession(neverNull(data.newAccountData).mailAddress, neverNull(data.newAccountData).password, 1 /* SessionType.Temporary */)
                .then((newSessionData) => newSessionData.credentials);
        }
        login
            .then(() => {
            if (!data.accountingInfo || !data.customer) {
                return locator.logins
                    .getUserController()
                    .loadCustomer()
                    .then((customer) => {
                    data.customer = customer;
                    return locator.logins.getUserController().loadCustomerInfo();
                })
                    .then((customerInfo) => locator.entityClient.load(AccountingInfoTypeRef, customerInfo.accountingInfo).then((accountingInfo) => {
                    data.accountingInfo = accountingInfo;
                }));
            }
        })
            .then(() => {
            this._invoiceDataInput = new InvoiceDataInput(data.options.businessUse(), data.invoiceData, InvoiceDataInputLocation.InWizard);
            let payPalRequestUrl = getLazyLoadedPayPalUrl();
            if (locator.logins.isUserLoggedIn()) {
                locator.logins.waitForFullLogin().then(() => payPalRequestUrl.getAsync());
            }
            this._paymentMethodInput = new PaymentMethodInput(data.options, this._invoiceDataInput.selectedCountry, neverNull(data.accountingInfo), payPalRequestUrl);
            this._availablePaymentMethods = this._paymentMethodInput.getVisiblePaymentMethods();
            this._selectedPaymentMethod(data.paymentData.paymentMethod);
            this._paymentMethodInput.updatePaymentMethod(data.paymentData.paymentMethod, data.paymentData);
        });
    }
    view(vnode) {
        const a = vnode.attrs;
        const onNextClick = () => {
            const invoiceDataInput = assertNotNull(this._invoiceDataInput);
            const paymentMethodInput = assertNotNull(this._paymentMethodInput);
            let error = invoiceDataInput.validateInvoiceData() || paymentMethodInput.validatePaymentData();
            if (error) {
                return Dialog.message(error).then(() => null);
            }
            else {
                a.data.invoiceData = invoiceDataInput.getInvoiceData();
                a.data.paymentData = paymentMethodInput.getPaymentData();
                return showProgressDialog("updatePaymentDataBusy_msg", Promise.resolve()
                    .then(() => {
                    let customer = neverNull(a.data.customer);
                    if (customer.businessUse !== a.data.options.businessUse()) {
                        customer.businessUse = a.data.options.businessUse();
                        return locator.entityClient.update(customer);
                    }
                })
                    .then(() => updatePaymentData(a.data.options.paymentInterval(), a.data.invoiceData, a.data.paymentData, null, a.data.upgradeType === "Signup" /* UpgradeType.Signup */, a.data.price, neverNull(a.data.accountingInfo)).then((success) => {
                    if (success) {
                        // Payment method confirmation (click on next), send selected payment method as an enum
                        const paymentMethodConfirmationStage = this.__signupPaidTest?.getStage(4);
                        paymentMethodConfirmationStage?.setMetric({
                            name: "paymentMethod",
                            value: PaymentMethodTypeToName[a.data.paymentData.paymentMethod],
                        });
                        paymentMethodConfirmationStage?.complete();
                        emitWizardEvent(this.dom, "showNextWizardDialogPage" /* WizardEventType.SHOW_NEXT_PAGE */);
                    }
                })));
            }
        };
        return m("#upgrade-account-dialog.pt", this._availablePaymentMethods
            ? [
                m(SegmentControl, {
                    items: this._availablePaymentMethods,
                    selectedValue: this._selectedPaymentMethod(),
                    onValueSelected: this._selectedPaymentMethod,
                }),
                m(".flex-space-around.flex-wrap.pt", [
                    m(".flex-grow-shrink-half.plr-l", {
                        style: {
                            minWidth: "260px",
                        },
                    }, m(neverNull(this._invoiceDataInput))),
                    m(".flex-grow-shrink-half.plr-l", {
                        style: {
                            minWidth: "260px",
                        },
                    }, m(neverNull(this._paymentMethodInput))),
                ]),
                m(".flex-center.full-width.pt-l", m("", {
                    style: {
                        width: "260px",
                    },
                }, m(LoginButton, {
                    label: "next_action",
                    onclick: onNextClick,
                }))),
            ]
            : null);
    }
}
export class InvoiceAndPaymentDataPageAttrs {
    constructor(upgradeData) {
        this._enabled = () => true;
        this.data = upgradeData;
    }
    nextAction(showErrorDialog) {
        return Promise.resolve(true);
    }
    headerTitle() {
        return lang.get("adminPayment_action");
    }
    isSkipAvailable() {
        return false;
    }
    isEnabled() {
        return this._enabled();
    }
    /**
     * Set the enabled function for isEnabled
     * @param enabled
     */
    setEnabledFunction(enabled) {
        this._enabled = enabled;
    }
}
export async function updatePaymentData(paymentInterval, invoiceData, paymentData, confirmedCountry, isSignup, price, accountingInfo) {
    const paymentResult = await locator.customerFacade.updatePaymentData(paymentInterval, invoiceData, paymentData, confirmedCountry);
    const statusCode = paymentResult.result;
    if (statusCode === "0" /* PaymentDataResultType.OK */) {
        // show dialog
        let braintree3ds = paymentResult.braintree3dsRequest;
        if (braintree3ds) {
            return verifyCreditCard(accountingInfo, braintree3ds, price);
        }
        else {
            return true;
        }
    }
    else if (statusCode === "1" /* PaymentDataResultType.COUNTRY_MISMATCH */) {
        const countryName = invoiceData.country ? invoiceData.country.n : "";
        const confirmMessage = lang.get("confirmCountry_msg", {
            "{1}": countryName,
        });
        const confirmed = await Dialog.confirm(() => confirmMessage);
        if (confirmed) {
            return updatePaymentData(paymentInterval, invoiceData, paymentData, invoiceData.country, isSignup, price, accountingInfo); // add confirmed invoice country
        }
        else {
            return false;
        }
    }
    else if (statusCode === "2" /* PaymentDataResultType.INVALID_VATID_NUMBER */) {
        await Dialog.message(() => lang.get("invalidVatIdNumber_msg") + (isSignup ? " " + lang.get("accountWasStillCreated_msg") : ""));
    }
    else if (statusCode === "3" /* PaymentDataResultType.CREDIT_CARD_DECLINED */) {
        await Dialog.message(() => lang.get("creditCardDeclined_msg") + (isSignup ? " " + lang.get("accountWasStillCreated_msg") : ""));
    }
    else if (statusCode === "4" /* PaymentDataResultType.CREDIT_CARD_CVV_INVALID */) {
        await Dialog.message("creditCardCVVInvalid_msg");
    }
    else if (statusCode === "5" /* PaymentDataResultType.PAYMENT_PROVIDER_NOT_AVAILABLE */) {
        await Dialog.message(() => lang.get("paymentProviderNotAvailableError_msg") + (isSignup ? " " + lang.get("accountWasStillCreated_msg") : ""));
    }
    else if (statusCode === "7" /* PaymentDataResultType.OTHER_PAYMENT_ACCOUNT_REJECTED */) {
        await Dialog.message(() => lang.get("paymentAccountRejected_msg") + (isSignup ? " " + lang.get("accountWasStillCreated_msg") : ""));
    }
    else if (statusCode === "9" /* PaymentDataResultType.CREDIT_CARD_DATE_INVALID */) {
        await Dialog.message("creditCardExprationDateInvalid_msg");
    }
    else if (statusCode === "10" /* PaymentDataResultType.CREDIT_CARD_NUMBER_INVALID */) {
        await Dialog.message(() => lang.get("creditCardNumberInvalid_msg") + (isSignup ? " " + lang.get("accountWasStillCreated_msg") : ""));
    }
    else if (statusCode === "8" /* PaymentDataResultType.COULD_NOT_VERIFY_VATID */) {
        await Dialog.message(() => lang.get("invalidVatIdValidationFailed_msg") + (isSignup ? " " + lang.get("accountWasStillCreated_msg") : ""));
    }
    else if (statusCode === "11" /* PaymentDataResultType.CREDIT_CARD_VERIFICATION_LIMIT_REACHED */) {
        await Dialog.message(() => lang.get("creditCardVerificationLimitReached_msg") + (isSignup ? " " + lang.get("accountWasStillCreated_msg") : ""));
    }
    else {
        await Dialog.message(() => lang.get("otherPaymentProviderError_msg") + (isSignup ? " " + lang.get("accountWasStillCreated_msg") : ""));
    }
    return false;
}
/**
 * Displays a progress dialog that allows to cancel the verification and opens a new window to do the actual verification with the bank.
 */
function verifyCreditCard(accountingInfo, braintree3ds, price) {
    return locator.entityClient.load(InvoiceInfoTypeRef, neverNull(accountingInfo.invoiceInfo)).then((invoiceInfo) => {
        let invoiceInfoWrapper = {
            invoiceInfo,
        };
        let resolve;
        let progressDialogPromise = new Promise((res) => (resolve = res));
        let progressDialog;
        const closeAction = () => {
            // user did not complete the 3ds dialog and PaymentDataService.POST was not invoked
            progressDialog.close();
            setTimeout(() => resolve(false), DefaultAnimationTime);
        };
        progressDialog = new Dialog("Alert" /* DialogType.Alert */, {
            view: () => [
                m(".dialog-contentButtonsBottom.text-break.selectable", lang.get("creditCardPendingVerification_msg")),
                m(".flex-center.dialog-buttons", m(Button, {
                    label: "cancel_action",
                    click: closeAction,
                    type: "primary" /* ButtonType.Primary */,
                })),
            ],
        })
            .setCloseHandler(closeAction)
            .addShortcut({
            key: Keys.RETURN,
            shift: false,
            exec: closeAction,
            help: "close_alt",
        })
            .addShortcut({
            key: Keys.ESC,
            shift: false,
            exec: closeAction,
            help: "close_alt",
        });
        let entityEventListener = (updates, eventOwnerGroupId) => {
            return promiseMap(updates, (update) => {
                if (isUpdateForTypeRef(InvoiceInfoTypeRef, update)) {
                    return locator.entityClient.load(InvoiceInfoTypeRef, update.instanceId).then((invoiceInfo) => {
                        invoiceInfoWrapper.invoiceInfo = invoiceInfo;
                        if (!invoiceInfo.paymentErrorInfo) {
                            // user successfully verified the card
                            progressDialog.close();
                            resolve(true);
                        }
                        else if (invoiceInfo.paymentErrorInfo && invoiceInfo.paymentErrorInfo.errorCode === "card.3ds2_pending") {
                            // keep waiting. this error code is set before starting the 3DS2 verification and we just received the event very late
                        }
                        else if (invoiceInfo.paymentErrorInfo && invoiceInfo.paymentErrorInfo.errorCode !== null) {
                            // verification error during 3ds verification
                            let error = "3dsFailedOther";
                            switch (invoiceInfo.paymentErrorInfo.errorCode) {
                                case "card.cvv_invalid":
                                    error = "cvvInvalid";
                                    break;
                                case "card.number_invalid":
                                    error = "ccNumberInvalid";
                                    break;
                                case "card.date_invalid":
                                    error = "expirationDate";
                                    break;
                                case "card.insufficient_funds":
                                    error = "insufficientFunds";
                                    break;
                                case "card.expired_card":
                                    error = "cardExpired";
                                    break;
                                case "card.3ds2_failed":
                                    error = "3dsFailed";
                                    break;
                            }
                            Dialog.message(getPreconditionFailedPaymentMsg(invoiceInfo.paymentErrorInfo.errorCode));
                            resolve(false);
                            progressDialog.close();
                        }
                        m.redraw();
                    });
                }
            }).then(noOp);
        };
        locator.eventController.addEntityListener(entityEventListener);
        let params = `clientToken=${encodeURIComponent(braintree3ds.clientToken)}&nonce=${encodeURIComponent(braintree3ds.nonce)}&bin=${encodeURIComponent(braintree3ds.bin)}&price=${encodeURIComponent(price)}&message=${encodeURIComponent(lang.get("creditCardVerification_msg"))}&clientType=${getClientType()}`;
        Dialog.message("creditCardVerificationNeededPopup_msg").then(() => {
            const paymentUrlString = locator.domainConfigProvider().getCurrentDomainConfig().paymentUrl;
            const paymentUrl = new URL(paymentUrlString);
            paymentUrl.hash += params;
            window.open(paymentUrl);
            progressDialog.show();
        });
        return progressDialogPromise.finally(() => locator.eventController.removeEntityListener(entityEventListener));
    });
}
//# sourceMappingURL=InvoiceAndPaymentDataPage.js.map