import { KyberEncapsulation, KyberKeyPair, KyberPrivateKey, KyberPublicKey } from "./KyberKeyPair.js";
import { Randomizer } from "../../random/Randomizer.js";
/**
 * Number of random bytes required for a Kyber operation
 */
export declare const KYBER_RAND_AMOUNT_OF_ENTROPY = 64;
export declare const KYBER_POLYVECBYTES: number;
export declare const KYBER_SYMBYTES = 32;
/**
 * @returns a new random kyber key pair.
 */
export declare function generateKeyPair(kyberWasm: WebAssembly.Exports, randomizer: Randomizer): KyberKeyPair;
/**
 * @param kyberWasm the WebAssembly module that implements our kyber primitives (liboqs)
 * @param publicKey the public key to encapsulate with
 * @param randomizer our randomizer that is used to the native library with entropy
 * @return the plaintext secret key and the encapsulated key for use with AES or as input to a KDF
 */
export declare function encapsulate(kyberWasm: WebAssembly.Exports, publicKey: KyberPublicKey, randomizer: Randomizer): KyberEncapsulation;
/**
 * @param kyberWasm the WebAssembly module that implements our kyber primitives (liboqs)
 * @param privateKey      the corresponding private key of the public key with which the encapsulatedKey was encapsulated with
 * @param ciphertext the ciphertext output of encapsulate()
 * @return the plaintext secret key
 */
export declare function decapsulate(kyberWasm: WebAssembly.Exports, privateKey: KyberPrivateKey, ciphertext: Uint8Array): Uint8Array;
